@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
 __title__ = 'metzoo-python-reader'
-__version__ = '0.0.13'
-__build__ = 0x00000D
+__version__ = '0.0.15'
+__build__ = 0x00000F
 __author__ = 'Edrans S.A.'
 __copyright__ = 'Copyright 2016 Edrans S.A.'
 
