@@ -27,7 +27,7 @@ def init_logger(name, filename):
 # def init_logger(name, filename):
 #     LOG_FORMAT = "%(asctime)-15s %(levelname)9s  %(filename)s:%(lineno)s %(message)s"
 #     logging.basicConfig(filename=filename, level=logging.DEBUG, format=LOG_FORMAT)
-
+#
 #     logger = logging.getLogger(name)
 #     console_handler = logging.StreamHandler()
 #     console_handler.setLevel(logging.DEBUG)
@@ -90,42 +90,55 @@ def parseimport(import_stmt, logger):
         return (m.group(1), [])
 
 
-def load(package, import_stmt, logger):
-    logger.debug("loading '{}'".format(package))
-    import pip
-    pip.main([ "install"
-             , "--no-cache-dir"
-             , "--upgrade"
-             , "-i"
-             , "https://testpypi.python.org/pypi"
-             , package
-             ])
+def load_module(package, import_stmt, logger):
+    logger.debug("loading module '{}'".format(import_stmt))
 
     name, fromlist = parseimport(import_stmt, logger)
-
-    fromsize = len(fromlist)
-    if fromsize > 1:
+    if len(fromlist) > 1:
         raise ValueError("'{}' must have only 1 imported name as in 'from X import Y' not 'from X import Y, Z'".format(import_stmt))
 
-    logger.debug("importing as '{}'".format(import_stmt))
     module = __import__(name, globals(), locals(), fromlist, -1)
+    if len(fromlist) == 1:
+        if fromlist[0] in dir(module):
+            module = getattr(module, fromlist[0])
+        else:
+            raise ImportError("module '{}' not found in '{}'".format(fromlist[0], name))
 
-    if fromsize == 0:
-        return module
-    else:
-        return getattr(module, fromlist[0])
+    if "read" not in dir(module):
+        raise ImportError("function 'read' not found in {}".format(import_stmt))
+
+    return module
+
+
+def install_package(repository, package):
+    logger.debug("installing '{}'".format(package))
+    import pip
+    pip.main(["install", "--upgrade", "-i", repository, package])
+
+
+def load(repository, package, import_stmt, logger):
+    try:
+        module = load_module(package, import_stmt, logger)
+    except ImportError:
+        install_package(repository, package)
+        module = load_module(package, import_stmt, logger)
+
+    return module
 
 
 def init_plugins(config_dict, logger):
     logger.debug("loading plugins")
     plugins = []
     plugins_config = lib.config.get(config_dict, "plugins")
+    repository = lib.config.get(config_dict, "repository", "https://pypi.python.org/pypi")
     for plugin_name, plugin_config in plugins_config.iteritems():
         validate_config(plugin_name, plugin_config, logger, ["module", "module.package", "module.import"])
         package = lib.config.get(plugin_config, "module.package")
         import_stmt = lib.config.get(plugin_config, "module.import")
+        plugin_repository = lib.config.get(plugin_config, "module.repository")
         try:
-            module = load(package, import_stmt, logger)
+            package_repository = plugin_repository or repository
+            module = load(package_repository, package, import_stmt, logger)
             plugins.append(Plugin(plugin_name, plugin_config, module))
         except Exception as e:
             logger.error("can't load '{}': {}".format(plugin_name, e))
@@ -144,8 +157,9 @@ def read(config_file, db_file, log):
 
     for plugin in plugins:
         logger.debug("reading from plugin '{}'".format(plugin.name))
-        logger.debug("read {} from {}".format(plugin.module.read(plugin.config), plugin.name))
-        for timestamp, agent, metric, value in plugin.module.read(plugin.config):
+        read_data = plugin.module.read(plugin.config, logger)
+        logger.debug("read {} from {}".format(read_data, plugin.name))
+        for timestamp, agent, metric, value in read_data:
             logger.debug("{}: {}.{} = {}".format(timestamp, agent, metric, value))
             db.save(timestamp, agent, metric, value)
 
